unit USerpent;

interface

 type
    TDirection = (NORD,EST,SUD,OUEST);
    TPointeurAnneau = ^TAnneau;
    TAnneau = record
                x,y: integer;
                svt: TPointeurAnneau
                end;
    TSerpent = record
                tete, queue: TPointeurAnneau;
                direction: TDirection
                end;
    TPoint = (x,y);
    TCoord = array[TPoint] of integer;

 procedure creerSerpent(var s:TSerpent; const x,y: integer);
 procedure allongerSerpent(var s:TSerpent; const x,y:integer);
 procedure couperQueue(var s:TSerpent);
 procedure deplacerSerpent(var s:TSerpent; const dir:TDirection);
 function getTete(const s:TSerpent) : TCoord;
 function getQueue(const s:TSerpent) : TCoord;

implementation

 procedure creerSerpent(var s:TSerpent; const x,y:integer);
 begin
    new(s.tete);
    s.tete.x := x-1;
    s.tete.y := y;
    new(s.queue);
    s.queue.x := x-2;
    s.queue.y := y;
    s.queue.svt := s.tete;
    allongerSerpent(s,x,y);
    s.direction := EST;
 end;

 procedure allongerSerpent(var s:TSerpent; const x,y:integer);
 begin
    new(s.tete.svt);
    s.tete := s.tete.svt;
    s.tete.svt := nil;
    s.tete.x := x;
    s.tete.y := y;
 end;

 procedure couperQueue(var s:TSerpent);
 var poubelle: TPointeurAnneau;
 begin
    poubelle := s.queue;
    s.queue := s.queue.svt;
    dispose(poubelle);
 end;

 procedure deplacerSerpent(var s:TSerpent; const dir:TDirection);
 begin
    if (abs(ord(dir) - ord(s.direction)) <> 2) then begin
        case dir of
                NORD: allongerSerpent(s, s.tete.x, s.tete.y-1);
                 EST: allongerSerpent(s, s.tete.x+1, s.tete.y);
                 SUD: allongerSerpent(s, s.tete.x, s.tete.y+1);
               OUEST: allongerSerpent(s, s.tete.x-1, s.tete.y);
        end;
        s.direction := dir;
    end;
 end;

 function getTete(const s:TSerpent) : TCoord;
 var coord:TCoord;
 begin
    coord[x] := s.tete.x;
    coord[y] := s.tete.y;
    result := coord;
 end;

 function getQueue(const s:TSerpent) : TCoord;
 var coord:TCoord;
 begin
    coord[x] := s.queue.x;
    coord[y] := s.queue.y;
    result := coord;
 end;

end.
